﻿Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Text
Imports System.Windows.Forms
Imports System.Runtime.InteropServices
Imports System.Diagnostics
Imports System.Drawing.Printing
Imports System.Drawing.Text

Partial Public Class GlassForm

    Private m_glassMargins As DwmApi.MARGINS
    Private Enum RenderMode
        None
        EntireWindow
        TopWindow
        Region
    End Enum
    Private m_RenderMode As RenderMode
    Private m_blurRegion As Region

    Public Sub New()
        InitializeComponent()
        m_RenderMode = RenderMode.None
    End Sub

    Protected Overloads Overrides Sub WndProc(ByRef msg As Message)
        MyBase.WndProc(msg)
        Const WM_DWMCOMPOSITIONCHANGED As Integer = 798
        Const WM_NCHITTEST As Integer = 132
        Const HTCLIENT As Integer = 1

        Select Case msg.Msg
            Case WM_NCHITTEST
                If HTCLIENT = msg.Result.ToInt32() Then
                    Dim p As New Point()
                    p.X = (msg.LParam.ToInt32() And 65535)
                    p.Y = (msg.LParam.ToInt32() >> 16)
                    p = PointToClient(p)

                    If PointIsOnGlass(p) Then
                        msg.Result = New IntPtr(2)
                    End If
                End If
                Exit Select
            Case WM_DWMCOMPOSITIONCHANGED

                If Not DwmApi.DwmIsCompositionEnabled() Then
                    m_RenderMode = RenderMode.None
                    m_glassMargins = Nothing
                    If m_blurRegion IsNot Nothing Then
                        m_blurRegion.Dispose()
                        m_blurRegion = Nothing
                    End If
                End If
                Exit Select
        End Select
    End Sub

    Private Function PointIsOnGlass(ByVal p As Point) As Boolean
        Return m_glassMargins IsNot Nothing AndAlso (m_glassMargins.cyTopHeight <= 0 OrElse m_glassMargins.cyTopHeight > p.Y)
    End Function

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)
        'Hier wird gemalt
        If DwmApi.DwmIsCompositionEnabled() Then
            'Aero ist aktiviert
            Select Case m_RenderMode
                Case RenderMode.EntireWindow
                    'ganzes Fenster
                    'schwarz wird zu Glas!
                    e.Graphics.FillRectangle(Brushes.Black, Me.ClientRectangle)
                    Exit Select
                Case RenderMode.TopWindow
                    'halbes Fenster
                    e.Graphics.FillRectangle(Brushes.Black, Rectangle.FromLTRB(0, 0, Me.ClientRectangle.Width, m_glassMargins.cyTopHeight))
                    Exit Select
                Case RenderMode.Region
                    'Region
                    If m_blurRegion IsNot Nothing Then
                        e.Graphics.FillRegion(Brushes.Black, m_blurRegion)
                    End If
                    Exit Select
            End Select
            'Text weiß machen, weil schwarz zu Glas werden würde
            Using textBrush As New SolidBrush(Color.FromArgb(255, 255, 255, 255))
                e.Graphics.TextRenderingHint = TextRenderingHint.SingleBitPerPixelGridFit
                e.Graphics.DrawString("This is writing on glass", Me.Font, textBrush, 10, 10)
            End Using
        Else
            'Aero ist aus, alternatives Rendering erforderlich
            Using textBrush As New SolidBrush(Color.FromArgb(255, 0, 0, 0))
                e.Graphics.TextRenderingHint = TextRenderingHint.SingleBitPerPixelGridFit
                e.Graphics.DrawString("Aero ist aus, kann kein Glas machen!", Me.Font, textBrush, 10, 10)
            End Using
        End If
    End Sub

    Private Sub ResetDwmBlurBehind()
        If DwmApi.DwmIsCompositionEnabled() Then
            Dim bbhOff As New DwmApi.DWM_BLURBEHIND()
            bbhOff.dwFlags = DwmApi.DWM_BLURBEHIND.DWM_BB_ENABLE Or DwmApi.DWM_BLURBEHIND.DWM_BB_BLURREGION
            bbhOff.fEnable = False
            bbhOff.hRegionBlur = IntPtr.Zero
            DwmApi.DwmEnableBlurBehindWindow(Me.Handle, bbhOff)
        End If
    End Sub

    Private Sub OnClientArea(ByVal sender As Object, ByVal e As EventArgs) Handles Button1.Click
        'ganzes Fenster mit Glas füllen
        ResetDwmBlurBehind()

        m_glassMargins = New DwmApi.MARGINS(-1, 0, 0, 0)
        m_RenderMode = RenderMode.EntireWindow

        If DwmApi.DwmIsCompositionEnabled() Then
            DwmApi.DwmExtendFrameIntoClientArea(Me.Handle, m_glassMargins)
        End If

        Me.FormBorderStyle = System.Windows.Forms.FormBorderStyle.SizableToolWindow
        Me.Invalidate()
    End Sub

    Private Sub OnTopHalf(ByVal sender As Object, ByVal e As EventArgs) Handles Button2.Click
        'halbes Fenster verglasen
        ResetDwmBlurBehind()

        m_glassMargins = New DwmApi.MARGINS(0, 150, 0, 0)
        m_RenderMode = RenderMode.TopWindow

        If DwmApi.DwmIsCompositionEnabled() Then
            DwmApi.DwmExtendFrameIntoClientArea(Me.Handle, m_glassMargins)
        End If

        Me.FormBorderStyle = System.Windows.Forms.FormBorderStyle.SizableToolWindow

        Me.Invalidate()
    End Sub

    Private Sub OnGlassRegion(ByVal sender As Object, ByVal e As EventArgs) Handles Button3.Click
        'Glasregion erstellen
        ResetDwmBlurBehind()

        m_glassMargins = New DwmApi.MARGINS(0, 0, 0, 0)
        m_RenderMode = RenderMode.Region

        If DwmApi.DwmIsCompositionEnabled() Then
            Using g As Graphics = CreateGraphics()
                DwmApi.DwmExtendFrameIntoClientArea(Me.Handle, m_glassMargins)
                If m_blurRegion IsNot Nothing Then
                    m_blurRegion.Dispose()
                End If
                m_blurRegion = New Region(New Rectangle(50, 10, Me.ClientRectangle.Width - 100, 150))

                Dim bbh As New DwmApi.DWM_BLURBEHIND()
                bbh.dwFlags = DwmApi.DWM_BLURBEHIND.DWM_BB_ENABLE Or DwmApi.DWM_BLURBEHIND.DWM_BB_BLURREGION Or DwmApi.DWM_BLURBEHIND.DWM_BB_TRANSITIONONMAXIMIZED
                bbh.fEnable = True
                bbh.hRegionBlur = m_blurRegion.GetHrgn(g)
                bbh.fTransitionOnMaximized = False
                DwmApi.DwmEnableBlurBehindWindow(Me.Handle, bbh)
            End Using
        End If

        Me.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None

        Me.Invalidate()
    End Sub

    Private Sub CompositionCheckBox_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs) Handles CompositionCheckBox.CheckedChanged
        'Aero systemweit ein-/ausschalten
        DwmApi.DwmEnableComposition(Not Me.CompositionCheckBox.Checked)
        Me.Invalidate()
    End Sub

End Class